require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchautodiscovervirtualdirectory',
  dscmeta_resource_friendly_name: 'xExchAutodiscoverVirtualDirectory',
  dscmeta_resource_name: 'MSFT_xExchAutodiscoverVirtualDirectory',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchAutodiscoverVirtualDirectory resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_allowservicerestart: {
      type: 'Optional[Boolean]',
      desc: 'Whether it is OK to recycle the app pool after making changes. Defaults to $true.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_extendedprotectionspnlist: {
      type: 'Optional[Array[String]]',
      desc: "Specifies a list of valid Service Principal Names (SPNs) if you're using Extended Protection for Authentication on the virtual directory.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_wssecurityauthentication: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether WS-Security (Web Services Security) authentication is enabled on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_windowsauthentication: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether Integrated Windows authentication is enabled on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_oauthauthentication: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether OAuth authentication is enabled on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_basicauthentication: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether Basic authentication is enabled on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_identity: {
      type: 'String',
      desc: 'The Identity parameter specifies the Autodiscover virtual directory that you want to modify.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_extendedprotectiontokenchecking: {
      type: "Optional[Enum['None', 'Allow', 'Require']]",
      desc: 'Defines how you want to use Extended Protection for Authentication on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_extendedprotectionflags: {
      type: "Optional[Array[Enum['None', 'Proxy', 'NoServiceNameCheck', 'AllowDotlessSpn', 'ProxyCohosting']]]",
      desc: 'Specifies custom settings for Extended Protection for Authentication on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_digestauthentication: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether Digest authentication is enabled on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: "Specifies the domain controller that's used by this cmdlet to read data from or write data to Active Directory.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
