require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchautomountpoint',
  dscmeta_resource_friendly_name: 'xExchAutoMountPoint',
  dscmeta_resource_name: 'MSFT_xExchAutoMountPoint',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchAutoMountPoint resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_mindisksize: {
      type: 'Optional[String]',
      desc: "The minimum size of a disk to consider using. Defaults to none. Should be in a format like '1024MB' or '1TB'.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_unitsize: {
      type: 'Optional[String]',
      desc: 'The unit size to use when formatting the disk. Defaults to 64k.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_filesystem: {
      type: "Optional[Enum['NTFS', 'REFS']]",
      desc: 'The file system to use when formatting the volume. Defaults to NTFS.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensureexchangevolumemountpointislast: {
      type: 'Optional[Boolean]',
      desc: 'Whether the EXVOL mount point should be moved to be the last mount point listed on each disk. Defaults to $false.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_autodagvolumesrootfolderpath: {
      type: 'String',
      desc: 'The parent folder for Exchange volume mount point folders.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_createsubfolders: {
      type: 'Optional[Boolean]',
      desc: 'If $true, specifies that DBNAME.db and DBNAME.log subfolders should be automatically created underneath the ExchangeDatabase mount points. Defaults to $false.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_autodagdatabasesrootfolderpath: {
      type: 'String',
      desc: 'The parent folder for Exchange database mount point folders.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_identity: {
      type: 'String',
      desc: 'The name of the server. Not actually used for anything.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_volumeprefix: {
      type: 'Optional[String]',
      desc: 'The prefix to give to Exchange Volume folders. Defaults to EXVOL.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_partitioningscheme: {
      type: "Optional[Enum['MBR', 'GPT']]",
      desc: 'The partitioning scheme for the volume. Defaults to GPT.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sparevolumecount: {
      type: 'Integer[0, 4294967295]',
      desc: 'How many spare volumes will be available.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_disktodbmap: {
      type: 'Array[String]',
      desc: "An array of strings containing the databases for each disk. Databases on the same disk should be in the same string, and comma separated. Example: 'DB1,DB2','DB3,DB4'. This puts DB1 and DB2 on one disk, and DB3 and DB4 on another.",

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
  },
)
