require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchecpvirtualdirectory',
  dscmeta_resource_friendly_name: 'xExchEcpVirtualDirectory',
  dscmeta_resource_name: 'MSFT_xExchEcpVirtualDirectory',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchEcpVirtualDirectory resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_allowservicerestart: {
      type: 'Optional[Boolean]',
      desc: 'Whether it is OK to recycle the app pool after making changes. Defaults to $true.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_extendedprotectionspnlist: {
      type: 'Optional[Array[String]]',
      desc: "Specifies a list of valid Service Principal Names (SPNs) if you're using Extended Protection for Authentication on the virtual directory.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_owaoptionsenabled: {
      type: 'Optional[Boolean]',
      desc: 'Specifies that Outlook on the web Options is enabled for end users.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_externalurl: {
      type: 'Optional[String]',
      desc: "The ExternalURL parameter specifies the URL that's used to connect to the virtual directory from outside the firewall.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_windowsauthentication: {
      type: 'Optional[Boolean]',
      desc: 'The WindowsAuthentication parameter specifies whether Integrated Windows authentication is enabled on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_basicauthentication: {
      type: 'Optional[Boolean]',
      desc: 'The BasicAuthentication parameter specifies whether Basic authentication is enabled on the virtual directory. ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_internalurl: {
      type: 'Optional[String]',
      desc: "The InternalURL parameter specifies the URL that's used to connect to the virtual directory from inside the firewall.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_formsauthentication: {
      type: 'Optional[Boolean]',
      desc: 'The FormsAuthentication parameter specifies whether forms-based authentication is enabled on the ECP virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_identity: {
      type: 'String',
      desc: 'The Identity of the ECP Virtual Directory.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_gziplevel: {
      type: "Optional[Enum['Off', 'Low', 'High', 'Error']]",
      desc: 'The GzipLevel parameter sets Gzip configuration information for the ECP virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_externalauthenticationmethods: {
      type: 'Optional[Array[String]]',
      desc: 'The ExternalAuthenticationMethods parameter specifies the authentication methods supported on the Exchange server from outside the firewall.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_extendedprotectiontokenchecking: {
      type: "Optional[Enum['None', 'Allow', 'Require']]",
      desc: 'Defines how you want to use Extended Protection for Authentication on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_extendedprotectionflags: {
      type: "Optional[Array[Enum['None', 'Proxy', 'NoServiceNameCheck', 'AllowDotlessSpn', 'ProxyCohosting']]]",
      desc: 'Specifies custom settings for Extended Protection for Authentication on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_digestauthentication: {
      type: 'Optional[Boolean]',
      desc: 'The DigestAuthentication parameter specifies whether Digest authentication is enabled on the virtual directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: "The DomainController parameter specifies the domain controller that's used by this cmdlet to read data from or write data to Active Directory. You identify the domain controller by its fully qualified domain name (FQDN). For example, dc01.contoso.com.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_adminenabled: {
      type: 'Optional[Boolean]',
      desc: "The AdminEnabled parameter specifies that the EAC isn't able to be accessed through the Internet.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_adfsauthentication: {
      type: 'Optional[Boolean]',
      desc: 'The AdfsAuthentication parameter specifies that the ECP virtual directory allows users to authenticate through Active Directory Federation Services (AD FS) authentication. This parameter accepts $true or $false. The default value is $false.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
