require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchexchangeserver',
  dscmeta_resource_friendly_name: 'xExchExchangeServer',
  dscmeta_resource_name: 'MSFT_xExchExchangeServer',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchExchangeServer resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_allowservicerestart: {
      type: 'Optional[Boolean]',
      desc: 'Whether it is OK to restart the Information Store service after licensing the server. Defaults to $false.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_staticglobalcatalogs: {
      type: 'Optional[Array[String]]',
      desc: 'The StaticGlobalCatalogs parameter specifies whether to configure a list of global catalogs to be used by the server via DSAccess.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_monitoringgroup: {
      type: 'Optional[String]',
      desc: 'The MonitoringGroup parameter specifies how to add your Exchange servers to monitoring groups. You can add your servers to an existing group or create a monitoring group based on location or deployment, or to partition monitoring responsibility among your servers.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_customerfeedbackenabled: {
      type: 'Optional[Boolean]',
      desc: 'The CustomerFeedbackEnabled parameter specifies whether the Exchange server is enrolled in the Microsoft Customer Experience Improvement Program (CEIP). The CEIP collects anonymous information about how you use Exchange and problems that you might encounter. If you decide not to participate in the CEIP, the servers are opted-out automatically.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_staticconfigdomaincontroller: {
      type: 'Optional[String]',
      desc: 'The StaticConfigDomainController parameter specifies whether to configure a domain controller to be used by the server via Directory Service Access (DSAccess).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_errorreportingenabled: {
      type: 'Optional[Boolean]',
      desc: 'The ErrorReportingEnabled parameter specifies whether error reporting is enabled.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_workloadmanagementpolicy: {
      type: 'Optional[String]',
      desc: 'The *-ResourcePolicy, *-WorkloadManagementPolicy and *-WorkloadPolicy system workload management cmdlets have been deprecated. System workload management settings should be customized only under the direction of Microsoft Customer Service and Support.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_identity: {
      type: 'String',
      desc: 'The Identity parameter specifies the GUID, distinguished name (DN), or name of the server.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_staticexcludeddomaincontrollers: {
      type: 'Optional[Array[String]]',
      desc: 'The StaticExcludedDomainControllers parameter specifies whether to exclude a list of domain controllers from being used by the server.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_productkey: {
      type: 'Optional[String]',
      desc: 'The ProductKey parameter specifies the server product key.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_internetwebproxybypasslist: {
      type: 'Optional[Array[String]]',
      desc: 'The InternetWebProxyBypassList parameter specifies a list of servers that bypass the web proxy server specified by the InternetWebProxy parameter. You identify the servers by their FQDN (for example, server01.contoso.com).',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_internetwebproxy: {
      type: 'Optional[String]',
      desc: 'The InternetWebProxy parameter specifies the web proxy server that the Exchange server uses to reach the internet. A valid value for this parameter is the URL of the web proxy server.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: "The DomainController parameter specifies the domain controller that's used by this cmdlet to read data from or write data to Active Directory. You identify the domain controller by its fully qualified domain name (FQDN). For example, dc01.contoso.com.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_staticdomaincontrollers: {
      type: 'Optional[Array[String]]',
      desc: 'The StaticDomainControllers parameter specifies whether to configure a list of domain controllers to be used by the server via DSAccess.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
  },
)
