require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchmailboxdatabase',
  dscmeta_resource_friendly_name: 'xExchMailboxDatabase',
  dscmeta_resource_name: 'MSFT_xExchMailboxDatabase',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchMailboxDatabase resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_autodagexcludefrommonitoring: {
      type: 'Optional[Boolean]',
      desc: 'The AutoDagExcludedFromMonitoringparameter specifies whether to exclude the mailbox database from the ServerOneCopyMonitor, which alerts an administrator when a replicated database has only one healthy copy available.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_logfolderpath: {
      type: 'String',
      desc: 'The LogFolderPath parameter specifies the folder location for log files.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_retaindeleteditemsuntilbackup: {
      type: 'Optional[Boolean]',
      desc: 'The RetainDeletedItemsUntilBackup parameter specifies whether to keep items in the Recoverable Items\Deletions folder of the mailbox until the next database backup occurs.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_adserversettingspreferredserver: {
      type: 'Optional[String]',
      desc: 'An optional domain controller to pass to Set-AdServerSettings -PreferredServer',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_offlineaddressbook: {
      type: 'Optional[String]',
      desc: "The OfflineAddressBook parameter specifies the offline address book that's associated with the mailbox database.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_recoverableitemswarningquota: {
      type: 'Optional[String]',
      desc: 'The RecoverableItemsWarningQuota parameter specifies the warning threshold for the size of the Recoverable Items folder for the mailbox.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_isexcludedfromprovisioningduetologicalcorruption: {
      type: 'Optional[Boolean]',
      desc: 'The IsExcludedFromProvisioningDueToLogicalCorruption parameter specifies whether to exclude the database from the mailbox provisioning load balancer that distributes new mailboxes randomly and evenly across the available databases.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_recoverableitemsquota: {
      type: 'Optional[String]',
      desc: 'The RecoverableItemsQuota parameter specifies the maximum size for the Recoverable Items folder of the mailbox.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_issuewarningquota: {
      type: 'Optional[String]',
      desc: 'The IssueWarningQuota parameter specifies the warning threshold for the size of the mailbox.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_journalrecipient: {
      type: 'Optional[String]',
      desc: 'The JournalRecipient parameter specifies the journal recipient to use for per-database journaling for all mailboxes on the database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_isexcludedfromprovisioning: {
      type: 'Optional[Boolean]',
      desc: 'The IsExcludedFromProvisioning parameter specifies whether to exclude the database from the mailbox provisioning load balancer that distributes new mailboxes randomly and evenly across the available databases.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_prohibitsendquota: {
      type: 'Optional[String]',
      desc: "The ProhibitSendQuota parameter specifies a size limit for the mailbox. If the mailbox reaches or exceeds this size, the mailbox can't send new messages, and the user receives a descriptive warning message.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_datamovereplicationconstraint: {
      type: "Optional[Enum['None', 'SecondCopy', 'SecondDatacenter', 'AllDatacenters', 'AllCopies']]",
      desc: 'The DataMoveReplicationConstraint parameter specifies the throttling behavior for high availability mailbox moves.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_server: {
      type: 'String',
      desc: 'The Server parameter specifies the server on which you want to create the database.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_backgrounddatabasemaintenance: {
      type: 'Optional[Boolean]',
      desc: 'The BackgroundDatabaseMaintenance parameter specifies whether the Extensible Storage Engine (ESE) performs database maintenance.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_allowservicerestart: {
      type: 'Optional[Boolean]',
      desc: 'Whether it is okay to restart the Information Store Service after adding a database. Defaults to $false.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_calendarloggingquota: {
      type: 'Optional[String]',
      desc: 'The CalendarLoggingQuota parameter specifies the maximum size of the log in the Recoverable Items folder of the mailbox that stores changes to calendar items.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_isexcludedfromprovisioningreason: {
      type: 'Optional[String]',
      desc: 'The IsExcludedFromProvisioningReason parameter specifies the reason why you excluded the mailbox database from the mailbox provisioning load balancer.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_indexenabled: {
      type: 'Optional[Boolean]',
      desc: 'The IndexEnabled parameter specifies whether Exchange Search indexes the mailbox database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_issuspendedfromprovisioning: {
      type: 'Optional[Boolean]',
      desc: 'The IsSuspendedFromProvisioning parameter specifies whether to exclude the database from the mailbox provisioning load balancer that distributes new mailboxes randomly and evenly across the available databases.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_circularloggingenabled: {
      type: 'Optional[Boolean]',
      desc: 'The CircularLoggingEnabled parameter specifies whether circular logging is enabled for the database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_allowfilerestore: {
      type: 'Optional[Boolean]',
      desc: 'The AllowFileRestore parameter specifies whether to allow a database to be restored from a backup.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_prohibitsendreceivequota: {
      type: 'Optional[String]',
      desc: "The ProhibitSendReceiveQuota parameter specifies a size limit for the mailbox. If the mailbox reaches or exceeds this size, the mailbox can't send or receive new messages. Messages sent to the mailbox are returned to the sender with a descriptive error message. This value effectively determines the maximum size of the mailbox.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: "The DomainController parameter specifies the domain controller that's used by this cmdlet to read data from or write data to Active Directory. You identify the domain controller by its fully qualified domain name (FQDN). For example, dc01.contoso.com.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_mailboxretention: {
      type: 'Optional[String]',
      desc: 'The MailboxRetention parameter specifies the length of time to keep deleted mailboxes before they are permanently deleted or purged.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_metacachedatabasemaxcapacityinbytes: {
      type: 'Optional[Integer[-9223372036854775808, 9223372036854775807]]',
      desc: 'The MetaCacheDatabaseMaxCapacityInBytes parameter specifies the size of the metacache database in bytes. To convert gigabytes to bytes, multiply the value by 1024^3. For terabytes to bytes, multiply by 1024^4.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt64',
      mof_is_embedded: false,
    },
    dsc_skipinitialdatabasemount: {
      type: 'Optional[Boolean]',
      desc: 'Whether the initial mount of databases should be skipped after database creation',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_eventhistoryretentionperiod: {
      type: 'Optional[String]',
      desc: 'The EventHistoryRetentionPeriod parameter specifies the length of time to keep event data.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_mountatstartup: {
      type: 'Optional[Boolean]',
      desc: 'The MountAtStartup parameter specifies whether to mount the mailbox database when the Microsoft Exchange Information Store service starts.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_databasecopycount: {
      type: 'Integer[0, 4294967295]',
      desc: 'The number of copies that the database will have once fully configured',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
    dsc_edbfilepath: {
      type: 'String',
      desc: 'The EdbFilePath parameter specifies the path to the database files.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'The Name parameter specifies the unique name of the mailbox database.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_deleteditemretention: {
      type: 'Optional[String]',
      desc: 'The DeletedItemRetention parameter specifies the length of time to keep deleted items in the Recoverable Items\Deletions folder in mailboxes.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_isexcludedfromprovisioningbyoperator: {
      type: 'Optional[Boolean]',
      desc: 'The IIsExcludedFromProvisioningByOperator parameter specifies whether to exclude the database from the mailbox provisioning load balancer that distributes new mailboxes randomly and evenly across the available databases.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
