require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchmailboxdatabasecopy',
  dscmeta_resource_friendly_name: 'xExchMailboxDatabaseCopy',
  dscmeta_resource_name: 'MSFT_xExchMailboxDatabaseCopy',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchMailboxDatabaseCopy resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_identity: {
      type: 'String',
      desc: 'The Identity parameter specifies the name of the database whose copy is being modified.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_truncationlagtime: {
      type: 'Optional[String]',
      desc: 'The TruncationLagTime parameter specifies the amount of time that the Microsoft Exchange Replication service should wait before truncating log files that have replayed into the passive copy of the database.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: "The DomainController parameter specifies the domain controller that's used by this cmdlet to read data from or write data to Active Directory. You identify the domain controller by its fully qualified domain name (FQDN). For example, dc01.contoso.com.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_replaylagtime: {
      type: 'Optional[String]',
      desc: 'The ReplayLagTime parameter specifies the amount of time that the Microsoft Exchange Replication service should wait before replaying log files that have been copied to the passive database copy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowservicerestart: {
      type: 'Optional[Boolean]',
      desc: 'Whether it is OK to restart Information Store after adding copy.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_adserversettingspreferredserver: {
      type: 'Optional[String]',
      desc: 'An optional domain controller to pass to Set-AdServerSettings -PreferredServer.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_seedingpostponed: {
      type: 'Optional[Boolean]',
      desc: "The SeedingPostponed switch specifies that the task doesn't seed the database copy, so you need to explicitly seed the database copy.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_replaylagmaxdelay: {
      type: 'Optional[String]',
      desc: 'The ReplayLagMaxDelay parameter specifies the maximum delay for lagged database copy play down.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_mailboxserver: {
      type: 'String',
      desc: 'The MailboxServer parameter specifies the name of the server that will host the database copy.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_activationpreference: {
      type: 'Optional[Integer[0, 4294967295]]',
      desc: 'The ActivationPreference parameter value is used as part of Active Managers best copy selection process and to redistribute active mailbox databases throughout the database availability group (DAG) when using the RedistributeActiveDatabases.ps1 script.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'UInt32',
      mof_is_embedded: false,
    },
  },
)
