require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchmaintenancemode',
  dscmeta_resource_friendly_name: 'xExchMaintenanceMode',
  dscmeta_resource_name: 'MSFT_xExchMaintenanceMode',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchMaintenanceMode resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_setinactivecomponentsfromanyrequestertoactive: {
      type: 'Optional[Boolean]',
      desc: 'Whether components that were set to Inactive by outside Requesters should also be set to Active when exiting Maintenance Mode. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_skiphealthchecks: {
      type: 'Optional[Boolean]',
      desc: 'Used when moving databases back to the server after taking the server out of maintenance mode. The SkipHealthChecks switch specifies whether to bypass passive copy health checks. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_skipmaximumactivedatabaseschecks: {
      type: 'Optional[Boolean]',
      desc: 'Used when moving databases back to the server after taking the server out of maintenance mode. The SkipMaximumActiveDatabasesChecks switch specifies whether to skip checking the value of MaximumPreferredActiveDatabases during the best copy and server selection (BCSS) process. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_skipactivecopychecks: {
      type: 'Optional[Boolean]',
      desc: "Used when moving databases back to the server after taking the server out of maintenance mode. The SkipActiveCopyChecks switch specifies whether to skip checking the current active copy to see if it's currently a seeding source for any passive databases. Defaults to False.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_skipclientexperiencechecks: {
      type: 'Optional[Boolean]',
      desc: 'Used when moving databases back to the server after taking the server out of maintenance mode. The SkipClientExperienceChecks switch specifies whether to skip the search catalog (content index) state check to see if the search catalog is healthy and up to date. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_upgradedserverversion: {
      type: 'Optional[String]',
      desc: 'Optional string to specify what the server version will be after applying a Cumulative Update. If the server is already at this version, requests to put the server in Maintenance Mode will be ignored. Version should be in the format ##.#.####.#, as in 15.0.1104.5.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_skipcpuchecks: {
      type: 'Optional[Boolean]',
      desc: 'Exchange 2016 Only. Used when moving databases back to the server after taking the server out of maintenance mode. The SkipCpuChecks switch specifies whether to skip the high CPU utilization checks. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_movepreferreddatabasesback: {
      type: 'Optional[Boolean]',
      desc: 'Whether to move back databases with an Activation Preference of one for this server after taking the server out of Maintenance Mode. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_skipmovesuppressionchecks: {
      type: 'Optional[Boolean]',
      desc: 'Exchange 2016 Only. Used when moving databases back to the server after taking the server out of maintenance mode. The SkipMoveSuppressionChecks switch specifies whether to skip the move suppression checks. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: 'The DomainController parameter specifies the fully qualified domain name (FQDN) of the domain controller that writes this configuration change to Active Directory.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_skipallchecks: {
      type: 'Optional[Boolean]',
      desc: 'Exchange 2016 Only. Used when moving databases back to the server after taking the server out of maintenance mode. The SkipAllChecks switch specifies whether to skip all checks. This switch is equivalent to specifying all of the individual skip parameters that are available on this cmdlet. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_additionalcomponentstoactivate: {
      type: 'Optional[Array[String]]',
      desc: 'When taking a server out of Maintenance Mode, the following components will be set to Active by default: ServerWideOffline, UMCallRouter, HighAvailability, Monitoring, RecoveryActionsEnabled. This parameter specifies an additional list of components to set to Active.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_mountdialoverride: {
      type: "Optional[Enum['None', 'Lossless', 'GoodAvailability', 'BestAvailability', 'BestEffort']]",
      desc: 'Used when moving databases back to the server after taking the server out of maintenance mode. The MountDialOverride parameter is used to override the auto database mount dial (AutoDatabaseMountDial) setting for the target server and specify an alternate setting. Defaults to None.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enabled: {
      type: 'Boolean',
      desc: 'Whether the server should be put into Maintenance Mode. When Enabled is set to True, the server will be put in Maintenance Mode. If False, the server will be taken out of Maintenance Mode.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_skiplagchecks: {
      type: 'Optional[Boolean]',
      desc: 'Used when moving databases back to the server after taking the server out of maintenance mode. The SkipLagChecks switch specifies whether to allow a copy to be activated that has replay and copy queues outside of the configured criteria. Defaults to False.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
