require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchumcallroutersettings',
  dscmeta_resource_friendly_name: 'xExchUMCallRouterSettings',
  dscmeta_resource_name: 'MSFT_xExchUMCallRouterSettings',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchUMCallRouterSettings resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_umstartupmode: {
      type: "Enum['TCP', 'TLS', 'Dual']",
      desc: 'UMStartupMode for the UM call router',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_server: {
      type: 'String',
      desc: 'Hostname of the UM server to configure',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_siptcplisteningport: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: "Specifies the TCP port that's used by the Microsoft Exchange Unified Messaging Call Router service to receive incoming calls",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_maxcallsallowed: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the maximum number of concurrent voice calls that the Unified Messaging service allows',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_ipaddressfamily: {
      type: "Optional[Enum['IPv4Only', 'IPv6Only', 'Any']]",
      desc: 'Specifies whether the UM IP gateway will use IPv4, IPv6, or both to communicate',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ipaddressfamilyconfigurable: {
      type: 'Optional[Boolean]',
      desc: "Specifies whether you're able to set the IPAddressFamily parameter to IPv6Only or Any",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_siptlslisteningport: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: "Specifies the Transport Layer Security (TLS) port that's used by the Microsoft Exchange Unified Messaging Call Router service to receive incoming calls",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: 'Optional Domain Controller to connect to',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dialplans: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies all dial plans that the Unified Messaging service handles incoming calls for',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
  },
)
