require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xexchumservice',
  dscmeta_resource_friendly_name: 'xExchUMService',
  dscmeta_resource_name: 'MSFT_xExchUMService',
  dscmeta_module_name: 'xExchange',
  dscmeta_module_version: '1.28.0.0',
  docs: 'The DSC xExchUMService resource type.
         Automatically generated from version 1.28.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_maxcallsallowed: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'Specifies the maximum number of concurrent voice calls that the Unified Messaging service allows',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_grammargenerationschedule: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies the Grammar Generation Schedule',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_irmlogmaxdirectorysize: {
      type: 'Optional[String]',
      desc: 'Specifies the maximum size of all IRM logs in the connectivity log directory. When a directory reaches its maximum file size, the server deletes the oldest log files first',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ipaddressfamily: {
      type: "Optional[Enum['IPv4Only', 'IPv6Only', 'Any']]",
      desc: 'Specifies whether the UM IP gateway will use IPv4, IPv6, or both to communicate',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_irmlogpath: {
      type: 'Optional[String]',
      desc: 'Specifies the default IRM log directory location',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credentials used to establish a remote PowerShell session to Exchange',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_irmlogenabled: {
      type: 'Optional[Boolean]',
      desc: 'Specifies whether to enable logging of Information Rights Management (IRM) transactions. IRM logging is enabled by default',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_identity: {
      type: 'String',
      desc: ' ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ipaddressfamilyconfigurable: {
      type: 'Optional[Boolean]',
      desc: "Specifies whether you're able to set the IPAddressFamily parameter to IPv6Only or Any",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_irmlogmaxfilesize: {
      type: 'Optional[String]',
      desc: 'Specifies the maximum size of each IRM log file. When a log file reaches its maximum file size, a new log file is created',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sipaccessservice: {
      type: 'Optional[String]',
      desc: 'Specifies the FQDN and TCP port of the nearest  Skype for Business Server pool location for inbound and outbound calls from remote Skype for Business users located outside of the network',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_irmlogmaxage: {
      type: 'Optional[String]',
      desc: 'Specifies the maximum age for the IRM log file. Log files that are older than the specified value are deleted',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_domaincontroller: {
      type: 'Optional[String]',
      desc: 'Optional Domain Controller to connect to',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_umstartupmode: {
      type: "Enum['TCP', 'TLS', 'Dual']",
      desc: 'UMStartupMode for the UM server',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_dialplans: {
      type: 'Optional[Array[String]]',
      desc: 'Specifies all dial plans that the Unified Messaging service handles incoming calls for',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
  },
)
